#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-12-18
#
# bakery plugin for check_mk
#
# 2021-12-23: reworked options handling
#

from pathlib import Path
from typing import List

from cmk.base.cee.plugins.bakery.bakery_api.v1 import FileGenerator, OS, Plugin, PluginConfig, register


def get_cve_2021_44228_log4j_files(conf: List[any]) -> FileGenerator:
    options_array: List = []
    search_path_array: List = []
    separator: str = ' '  # needs matching separator in the shell scripts

    options = conf[1].copy()

    interval = options.get('interval', 86400)
    timeout = options.get('timeout', 300)

    search_path = ''
    if conf[0] == 'linux':
        for path in options['search_linux']:
            path = path.strip(' ').strip("'").strip('"')
            search_path_array.append(f'"{path}"')
        search_path = ' '.join(search_path_array)
    elif conf[0] == 'windows':
        if options['search_windows'] == 'all_drives':
            options_array.append('--all-drives')
        elif 'drives_to_scan' in options['search_windows']:
            label, drives_to_scan = options['search_windows']
            drives_to_scan = f'--drives {",".join(drives_to_scan)}'
            options_array.append(drives_to_scan)
        elif 'search_paths' in options['search_windows']:
            label, search_path = options['search_windows']
            search_path = f'{" ".join(search_path)}'

    for path in options.get('exclude_paths', []):
        path = path.strip(' ').strip("'").strip('"')
        options_array.append(f'--exclude "{path}"')

    if options.get('exclude_fs'):
        exclude_fs = ','.join(options['exclude_fs'])
        options_array.append(f'--exclude-fs {exclude_fs}')

    if options.get('syslog'):
        options_array.append(f'--syslog-level {options["syslog"].get("syslog_level", "info")}')
        syslog_server = f'--syslog-udp {options["syslog"]["syslog_server"]}'
        syslog_port = options['syslog'].get('syslog_port', None)
        if syslog_port is not None:
            syslog_port = f':{syslog_port}'
        else:
            syslog_port = ''
        options_array.append(f'{syslog_server}{syslog_port}')

    if options.get('reporting'):
        report_dir = options["reporting"]["report_dir"].strip(' ').strip("'").strip('"')
        options_array.append(f'--report-dir "{report_dir}"')
        options_array.append(options['reporting'].get('report_format', '--report-csv'))
        if options['reporting'].get('no_empty_report'):
            options_array.append('--no-empty-report')

    if options.get('fix_files'):
        backup_dir = options["fix_files"]["backup_dir"].strip(' ').strip("'").strip('"')
        options_array.append(f'--backup-path "{backup_dir}"')
        if options['fix_files'].get('not_exclude_backup') is None:
            options_array.append(f'--exclude {backup_dir}')
        if options['fix_files'].get('force_fix'):
            options_array.append(f'--force-fix')

    for key in [
        'interval',
        'timeout',
        'search_linux',
        'search_windows',
        'exclude_paths',
        'exclude_fs',
        'syslog',
        'reporting',
        'fix_files',
    ]:
        try:
            options.pop(key)
        except KeyError:
            pass

    for value in options.values():
        options_array.append(value)

    options = separator.join(options_array)

    if conf[0] == 'linux':
        yield Plugin(
            base_os=OS.LINUX,
            source=Path('cve_2021_44228_log4j.linux'),
            target=Path('cve_2021_44228_log4j.sh'),
            asynchronous=True,
            interval=interval,
            timeout=timeout,
        )
        yield Plugin(
            base_os=OS.LINUX,
            source=Path('log4j2-scan.linux'),
            target=Path('../bin/log4j2-scan'),
        )

        yield PluginConfig(
            base_os=OS.LINUX,
            lines=[f'OPTIONS=({options}{separator}{search_path});'],
            target=Path('cve_2021_44228_log4j.cfg'),
            include_header=True,
        )

    elif conf[0] == 'windows':
        yield Plugin(
            base_os=OS.WINDOWS,
            source=Path('cve_2021_44228_log4j.windows'),
            target=Path('cve_2021_44228_log4j.ps1'),
            asynchronous=True,
            interval=interval,
            timeout=timeout,
        )
        yield Plugin(
            base_os=OS.WINDOWS,
            source=Path('log4j2-scan.windows'),
            target=Path('..\\bin\\log4j2-scan.exe'),
        )
        yield PluginConfig(
            base_os=OS.WINDOWS,
            lines=[f'OPTIONS={options}{separator}{search_path}'],
            target=Path('cve_2021_44228_log4j.cfg'),
            include_header=True,
        )


register.bakery_plugin(
    name='cve_2021_44228_log4j',
    files_function=get_cve_2021_44228_log4j_files,
)
